#!/usr/bin/awk -f
#
# Copyright (c) 2008
#      IS/AIST-ST2I/CNRS Joint Japanese-French Robotics Laboratory (JRL).
# All rights reserved.
#
# Redistribution  and  use in source   and binary forms,  with or without
# modification, are permitted provided that  the following conditions are
# met:
#
#   1. Redistributions  of  source code must  retain  the above copyright
#      notice, this list of conditions and the following disclaimer.
#   2. Redistributions in binary form must  reproduce the above copyright
#      notice,  this list of  conditions and  the following disclaimer in
#      the  documentation   and/or  other  materials   provided with  the
#      distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE  AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY  EXPRESS OR IMPLIED WARRANTIES, INCLUDING,  BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES   OF MERCHANTABILITY AND  FITNESS  FOR  A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO  EVENT SHALL THE AUTHOR OR  CONTRIBUTORS
# BE LIABLE FOR ANY DIRECT, INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING,  BUT  NOT LIMITED TO, PROCUREMENT  OF
# SUBSTITUTE  GOODS OR SERVICES;  LOSS   OF  USE,  DATA, OR PROFITS;   OR
# BUSINESS  INTERRUPTION) HOWEVER CAUSED AND  ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE  USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authored by Anthony Mallet on Wed Apr 30 2008.
#

# strtonum.awk reads a sphinx2 FSG file where states and transitions are
# described by names instead of numbers. It outputs a regular FSG grammar
# for sphinx2, where the name of the states are replaced by a number.
#

# --- usage() --------------------------------------------------------
#
function usage() {
  print "usage: strtonum.awk input.nfsg output.fsg" > "/dev/stderr"
}

BEGIN {
  if (ARGC != 3) {
    usage(); exit 2;
  }

  # get all state names
  while(getline < ARGV[1]) {
    if ($1 ~ /^START_STATE$/ && NF == 2) {
      start_state = $2;
    } else if ($1 ~ /^FINAL_STATE$/ && NF == 2) {
      final_state = $2;
    } else if ($1 ~ /^T(RANSITION)?$/ && (NF == 4 || NF == 5)) {
      states[$2] = 0;
      states[$3] = 0;
    }
  }

  # enumerate states 
  nstates = 0;
  for(s in states) states[s] = nstates++;

  print "Found " ns " states:";
  for(s in states) print s "\t=>\t" states[s];

  close(ARGV[1])

  # output new file
  while(getline < ARGV[1]) {
    if ($1 == "NUM_STATES" && NF == 2) {
      print $1 " " nstates > ARGV[2];
    } else if ($1 == "START_STATE" && NF == 2) {
      print $1 " " states[start_state] > ARGV[2];
    } else if ($1 == "FINAL_STATE" && NF == 2) {
      print $1 " " states[final_state] > ARGV[2];
    } else if ($1 ~ /^T(RANSITION)?$/ && (NF == 4 || NF == 5)) {
      print $1 " " states[$2] " " states[$3] " " $4 " " $5 > ARGV[2];
    } else {
      print $0 > ARGV[2];
    }
  }
}
